@@ -44,7 +44,7 @@ def logstring(element):
 
 DATE_ATTRIBUTES = {
                   'article.created', 'article_date_original',
-                  'article.published', 'article:published_time', 
+                  'article.published', 'article:published_time',
                   'bt:pubdate', 'citation_date', 'citation_publication_date',
                   'created', 'cxenseparse:recs:publishtime',
                   'date', 'date_published',
@@ -73,9 +73,9 @@ TRIM_REGEX = re.compile(r'[\n\r\s\t]+')
 NON_DIGITS_REGEX = re.compile(r'\D+$')
 GER_STRIP_REGEX = re.compile(r'^am ')
 
-LAST_MODIFIED = set(['lastmodified', 'last-modified'])
-ITEMPROP_ATTRS = set(['datecreated', 'datepublished', 'pubyear'])
-CLASS_ATTRS = set(['date-published', 'published', 'time published'])
+LAST_MODIFIED = {'lastmodified', 'last-modified'}
+ITEMPROP_ATTRS = {'datecreated', 'datepublished', 'pubyear'}
+CLASS_ATTRS = {'date-published', 'published', 'time published'}
 
 
 @lru_cache(maxsize=CACHE_SIZE)
@@ -539,9 +539,8 @@ def search_page(htmlstring, outputformat, original_date, min_date, max_date):
     bestmatch = search_pattern(htmlstring, SIMPLE_PATTERN, YEAR_PATTERN, YEAR_PATTERN, original_date, min_date, max_date)
     if bestmatch is not None:
         pagedate = '-'.join([bestmatch.group(0), '01', '01'])
-        if date_validator(pagedate, '%Y-%m-%d', latest=max_date) is True and (
-            copyear == 0 or int(bestmatch.group(0)) >= copyear
-        ):
+        if date_validator(pagedate, '%Y-%m-%d', latest=max_date) is True and \
+            int(bestmatch.group(0)) >= copyear:
             LOGGER.debug('date found for pattern "%s": %s', SIMPLE_PATTERN, pagedate)
             return convert_date(pagedate, '%Y-%m-%d', outputformat)
 
@@ -609,14 +608,10 @@ def find_date(htmlobject, extensive_search=True, original_date=False, outputform
             return dateresult
 
     # first, try header
-    try:
-        header_result = examine_header(tree, outputformat, extensive_search,
-                                       original_date, min_date, max_date)
-    except etree.XPathEvalError as err:
-        LOGGER.error('header XPath %s', err)
-    else:
-        if header_result is not None:
-            return header_result
+    header_result = examine_header(tree, outputformat, extensive_search,
+                                   original_date, min_date, max_date)
+    if header_result is not None:
+        return header_result
 
     # try to use JSON data
     json_result = json_search(tree, outputformat, original_date, min_date, max_date)
